@@ -1,9 +1,9 @@
 /********************************************************************************
    zfuncs.h      include file for zfuncs functions
 
-   Copyright 2007-2018 Michael Cornelison
+   Copyright 2007-2020 Michael Cornelison
    source code URL: https://kornelix.net
-   contact: kornelix@posteo.de
+   contact: mkornelix@gmail.com
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -17,7 +17,7 @@
 
 *********************************************************************************/
 
-//     zfuncs.h   version  v.7.0
+//     zfuncs.h   version  v.7.4  (for Fotoxx 20.0)
 
 #include <sys/sysinfo.h>
 #include <sys/time.h>
@@ -76,6 +76,7 @@
 #define  null   NULL
 #define  true   1
 #define  false  0
+#define  NOP
 
 //  trace execution: source file, function, line no, caller address
 
@@ -83,21 +84,20 @@
 
 //  system functions ============================================================
 
-void *zmalloc(uint cc);                                                          //  malloc() with counter
-void zfree(void *pp);                                                            //  free() with counter
-char *zstrdup(cchar *string, int addcc = 0);                                     //  strdup() with counter
+void *zmalloc(size_t cc);                                                        //  malloc() wrapper
+void zfree(void *pp);                                                            //  free() wrapper
+char *zstrdup(cchar *string, int addcc = 0);                                     //  strdup() wrapper with opt. expand
 void zmalloc_report();                                                           //  print statistics report
-
 void printz(cchar *format, ...);                                                 //  printf() with immediate fflush()
-void zpopup_message(int secs, cchar *format, ...);                               //  popup message, thread safe
-
+void zexit(cchar *message, ...);                                                 //  exit a process and kill child processes
 void zbacktrace();                                                               //  produce a backtrace to stdout
 void zappcrash(cchar *format, ...);                                              //  crash with popup message in text window
 void catch_signals();                                                            //  catch signals and do backtrace dump
 void trace(cchar *file, cchar *func, int line, void *addr);                      //  implements TRACE macro
 void tracedump();                                                                //  dump program trace data
 
-void beroot(int argc = 0, char *argv[] = 0);                                     //  restart image as root if password OK
+void beroot(int argc, char *argv[]);                                             //  restart program as root if password OK
+int  runroot(cchar *command);                                                    //  run command as root via su or sudo
 double get_seconds();                                                            //  seconds since 2000.01.01
 void start_timer(double &time0);                                                 //  start a timer
 double get_timer(double &time0);                                                 //  get elapsed time in seconds
@@ -117,6 +117,7 @@ int  coretemp();
 int  disktemp(char *disk);                                                       //  get disk temp, e.g. "/dev/sda"
 void zsleep(double dsecs);                                                       //  sleep specified seconds
 void zloop(double dsecs);                                                        //  loop specified seconds
+void make_global_lockfile(cchar *lockname, char **lockfile);                     //  create global lockfile from lock name
 int  global_lock(cchar *lockfile);                                               //  obtain exclusive lock, multi-process
 int  global_unlock(int fd, cchar *lockfile);                                     //  release the lock
 int  resource_lock(int &resource);                                               //  simple lock/unlock usable with GTK
@@ -128,6 +129,7 @@ void start_detached_thread(void * tfunc(void *), void * arg);
 pthread_t start_Jthread(void * threadfunc(void *), void * arg);                  //  start joinable thread function
 int  wait_Jthread(pthread_t tid);                                                //  wait for completion (join thread)
 void synch_threads(int NT = 0);                                                  //  synchronize NT threads
+void set_cpu_affinity(int cpu);                                                  //  set cpu affinity for calling thread
 int  zsystem(cchar *command);                                                    //  system() + return child process status
 int  shell_quiet(cchar *command, ...);                                           //  format/run shell command, return status
 int  shell_ack(cchar *command, ...);                                             //   ""  + popup an error message if error
@@ -137,14 +139,13 @@ char * command_output(int &contx, cchar *command, ...);
 int  command_status(int contx);                                                  //  get exit status of command
 int  command_kill(int contx);                                                    //  kill command before completion
 int  signalProc(cchar * pname, cchar * signal);                                  //  pause/resume/kill subprocess
-int  runroot(cchar *sucomm, cchar *command);                                     //  run command as root via su or sudo
 char * fgets_trim(char * buff, int maxcc, FILE *, int bf = 0);                   //  fgets + trim trailing \n \r (blanks)
-int  samedirk(cchar *file1, cchar *file2);                                       //  returns 1 if files in same directory
+int  samefolder(cchar *file1, cchar *file2);                                     //  returns 1 if files in same folder
 int  parsefile(cchar *path, char **dir, char **file, char **ext);                //  parse a filespec
 int  renamez(cchar *file1, cchar *file2);                                        //  rename, also across file systems
-int  check_create_dir(char *path);                                               //  check if directory exists, ask to create
-int  copyFile(cchar *sfile, cchar *dfile);                                       //  copy file to file or file to directory
-int  zreaddir(cchar *directory, char **&files);                                  //  return all files in a directory, sorted
+int  check_create_dir(char *path);                                               //  check if folder exists, ask to create
+int  copyFile(cchar *sfile, cchar *dfile);                                       //  copy file to file or file to folder
+int  zreaddir(cchar *folder, char **&files);                                     //  return all files in a folder, sorted
 char * combine_argvs(int argc, char *argv[], int Nth);                           //  combine argv[ii] elements from Nth to last
 char * zescape_quotes(cchar *file);                                              //  escape quote marks (") in file name
 
@@ -172,6 +173,7 @@ cchar * strField(cchar *string, cchar *delims, int Nth);
 cchar * strField(cchar *string, cchar delim, int Nth);                           //  get Nth delimited field in string
 int  strParms(int &bf, cchar *inp, char *pname, int maxcc, double &pval);        //  parse string: name1=val1 | name2 ...
 int  strHash(cchar *string, int max);                                            //  string --> random int 0 to max-1
+int64  strHash64(cchar *string, int64 max);                                      //  string --> random int 0 to max-1
 int  strncpy0(char *dest, cchar *source, uint cc);                               //  strncpy, insure null, return 0 if fit
 void strnPad(char *dest, cchar *source, int cc);                                 //  strncpy with blank padding to cc
 int  strTrim(char *dest, cchar *source);                                         //  remove trailing blanks
@@ -198,6 +200,8 @@ int  utf8substring(char *utf8out, cchar *utf8in, int pos, int cc);
 int  utf8_check(cchar *string);                                                  //  check utf8 string for encoding errors
 int  utf8_position(cchar *utf8in, int Nth);                                      //  get byte position of Nth graphic char.
 int  zsed(cchar *file, ...);                                                     //  replace string1/3... with string2/4...
+cchar * zstrstr(cchar *haystack, cchar *needle);                                 //  these work like strstr() and strcasestr()
+cchar * zstrcasestr(cchar *haystack, cchar *needle);                             //   but "" does NOT match any string.
 
 //  number conversion ===========================================================
 
@@ -287,33 +291,36 @@ void Qtext_close(Qtext *qtext);
 
 //  application initialization and administration ===============================
 
-int make_appimage_desktop(cchar *appname);                                       //  make appimage desktop and icon files
-void uninstall_appimage();                                                       //  uninstall desktop and icon files
+int appimage_install(cchar *appname);                                            //  make appimage desktop and icon files
+void appimage_unstall();                                                         //  uninstall appimage
 
-int zinitapp(cchar *appname, cchar *homedir = 0);                                //  initz. app directories and files
+int zinitapp(cchar *appvers, cchar *homedir = 0);                                //  initialize app (appname-N.N, homedir)
 
 cchar * get_zprefix();                                                           //  get /usr or /usr/local  ... 
 cchar * get_zhomedir();                                                          //  get /home/user/.appname/
-cchar * get_zdatadir();                                                          //  get data directory
-cchar * get_zimagedir();                                                         //  get image directory
-cchar * get_zdocdir();                                                           //  get document directory
-cchar * get_zicondir();                                                          //  get icon directory
+cchar * get_zdatadir();                                                          //  get data folder
+cchar * get_zimagedir();                                                         //  get image folder
+cchar * get_zdocdir();                                                           //  get document folder
 
 void zsetfont(cchar *newfont);                                                   //  set new app font and size
 int  widget_font_metrics(GtkWidget *widget, int &fww, int &fhh);                 //  get widget font char width/height
 
 int  locale_filespec(cchar *ftype, cchar *fname, char *filespec);                //  get a locale dependent file
-void showz_userguide(cchar *context = 0);                                        //  show user guide in new process
-void showz_logfile();                                                            //  show log file in popup window
+void showz_logfile(GtkWidget *parent);                                           //  show log file in popup window
 void showz_textfile(cchar *type, cchar *file, GtkWidget *parent);                //  show text file [.gz] in popup window
 void showz_html(cchar *url);                                                     //  show html via preferred browser
+void showz_docfile(GtkWidget *, cchar *docfile, cchar *topic);                   //  show docfile topic and assoc. image
+
+void appruns_update();                                                           //  update run counter in appruns file
+void phone_home();                                                               //  send usage statistics
+void phone_home_allow(GtkWidget *parent);                                        //  user dialog to allow/block phone_home()
 
 //  translation functions =======================================================
 
 #define E2Xmaxent 2000                                                           //  max. translation strings
 #define E2Xmaxcc 4000                                                            //  max. cc per string
 
-void E2Xinit(cchar *lang);                                                       //  setup for message translation
+void E2Xinit(cchar *lang, int Fdump);                                            //  setup for message translation
 cchar * E2X(cchar *english);                                                     //  get translation for English message
 cchar * E2X_missing(int &ftf);                                                   //  get missing translations, one per call
 
@@ -325,6 +332,9 @@ cchar * E2X_missing(int &ftf);
 void zmainloop(int skip = 0);                                                    //  do main loop, process menu events
 void zmainsleep(float secs);                                                     //  do main loop and sleep designated time
 
+
+/********************************************************************************/
+
 //  cairo drawing region for GDK window      GTK 3.21 version
 
 #if GTK_CHECK_VERSION(3,22,0)
@@ -363,15 +373,22 @@ void textwidget_insert(GtkWidget *widget, int bold, int line, cchar *format, ...
 void textwidget_replace(GtkWidget *widget, int bold, int line, cchar *format, ...);    //  replace line
 void textwidget_delete(GtkWidget *widget, int line);                                   //  delete line
 int  textwidget_find(GtkWidget *widget, char *matchtext, int line1);                   //  find matching line
-void textwidget_scroll(GtkWidget *widget, int line);                                   //  scroll to make line visible
+void textwidget_insert_pixbuf(GtkWidget *textwidget, int line, GdkPixbuf *pixbuf);     //  insert pixbuf image
+void textwidget_scroll(GtkWidget *widget, int line);                                   //  scroll line on screen
+void textwidget_scroll_top(GtkWidget *widget, int line);                               //  scroll line to top of window
+void textwidget_get_visible_lines(GtkWidget *textwidget, int &top, int &bott);         //  get range of visible lines
 void textwidget_dump(GtkWidget *widget, cchar *filename);                              //  dump all text into a file
 void textwidget_save(GtkWidget *widget, GtkWindow *parent);                            //  same, with save-as dialog
 char * textwidget_line(GtkWidget *widget, int line, int strip);                        //  retrieve line (strip \n) 
 void textwidget_highlight_line(GtkWidget *widget, int line);                           //  highlight line
 char * textwidget_word(GtkWidget *, int line, int posn, cchar *dlims, char &end);      //  retrieve word
 void textwidget_highlight_word(GtkWidget *widget, int line, int posn, int cc);         //  highlight word
-typedef void textwidget_callbackfunc_t(GtkWidget *, int line, int posn, int KBkey);    //  callback function to receive
-void textwidget_set_callbackfunc(GtkWidget *, textwidget_callbackfunc_t func);         //    clicked line/posn or KB press
+void textwidget_bold_word(GtkWidget *widget, int line, int posn, int cc);              //  make word bold
+void textwidget_underline_word(GtkWidget *widget, int line, int posn, int cc);         //  make word underlined
+void textwidget_font_attributes(GtkWidget *widget);                                    //  set font attributes for all text
+
+typedef void textwidget_callbackfunc_t(GtkWidget *, int line, int posn, int KBkey);    //  widget event function to receive
+void textwidget_set_eventfunc(GtkWidget *, textwidget_callbackfunc_t func);            //    mouse click and KB events
 
 
 /********************************************************************************/
@@ -424,7 +441,9 @@ struct vmenuent {
    cchar       *icon;                                                            //  opt. icon file name
    cchar       *desc;                                                            //  description (mouse hover popup)
    cbFunc      *func;                                                            //  callback func (GtkWidget *, cchar *arg)
-   cchar       *arg;                                                             //  callback arg
+   cbFunc      *RMfunc;                                                          //  alternate func for right mouse click
+   cchar       *arg;                                                             //  callback arg for func
+   cchar       *RMarg;                                                           //  callback arg for RMfunc
    PIXBUF      *pixbuf;                                                          //  icon pixbuf or null
    PangoLayout *playout1, *playout2;                                             //  normal and bold menu text
    int         namex, namey;                                                     //  menu name position in layout
@@ -448,6 +467,7 @@ Vmenu *Vmenu_new(GtkWidget *vbox, float fgRGB[3], float bgRGB[3]);
 void Vmenu_add(Vmenu *vbm, cchar *name, cchar *icon,                             //  add menu item with response function
                int iconww, int iconhh, cchar *desc,
                cbFunc func, cchar *arg);                                         //  function may be popup_menu()
+void Vmenu_add_RMfunc(Vmenu *vbm, int me, cbFunc RMfunc, cchar *arg);            //  add auto function if right mouse click
 void Vmenu_block(int flag);                                                      //  block or unblock menu
 
 
@@ -486,8 +506,10 @@ struct zdialog  {
       int         uniqueID;                        //  unique ID, monotone increasing
       char        *title;                          //  dialog title
       void        *eventCB;                        //  dialog event user callback function
+      void        *popup_report_CB;                //  callback function for popup_report
       int         zrunning;                        //  dialog is running (0,1)
       int         zstat;                           //  dialog status (from completion button)
+      char        zstat_button[40];                //  completion button label
       int         disabled;                        //  widget signals/events are disabled
       int         saveposn;                        //  save and recall window position each use
       int         saveinputs;                      //  save and recall user inputs each use
@@ -507,6 +529,8 @@ void zdialog_set_title(zdialog *zd, cchar *title);
 void zdialog_set_modal(zdialog *zd);                                             //  set zdialog modal
 void zdialog_set_decorated(zdialog *zd, int decorated);                          //  set zdialog decorated or not
 void zdialog_present(zdialog *zd);                                               //  zdialog visible and on top
+void zdialog_can_focus(zdialog *zd, int Fcan);                                   //  zdialog can/not have focus (e.g. report)
+void zdialog_set_focus(zdialog *zd, cchar *widget = null);                       //  set focus on window [ widget ]
 
 int zdialog_add_widget(zdialog *zd,                                              //  add widget to zdialog
       cchar *type, cchar *name, cchar *pname,                                    //  required args
@@ -517,7 +541,7 @@ int zdialog_add_widget(zdialog *zd,
       cchar *type, cchar *name, cchar *pname,                                    //  (alternative form)
       cchar *data, cchar *options);                                              //  "size=nn|homog|expand|space=nn|wrap"
 
-int zdialog_valid(zdialog *zd, int errmess = 1);                                 //  return 1/0 if zdialog valid/invalid
+int zdialog_valid(zdialog *zd, cchar *title = 0);                                //  return 1/0 if zdialog valid/invalid
 GtkWidget * zdialog_widget(zdialog *zd, cchar *name);                            //  GTK widget from zdialog widget name
 int zdialog_set_image(zdialog *zd, cchar *name, GdkPixbuf *);                    //  set "image" widget from a GDK pixbuf
 int zdialog_add_ttip(zdialog *zd, cchar *wname, cchar *ttip);                    //  add popup tool tip to a widget
@@ -561,7 +585,7 @@ int zdialog_cb_popup(zdialog *zd, cchar *name);
 int zdialog_cb_save(zdialog *zd, cchar *name, cchar *file);                      //  save combo box list to a file
 int zdialog_cb_load(zdialog *zd, cchar *name, cchar *file);                      //  load combo box list from a file
 
-int  zdialog_positions(cchar *action);                                           //  load or save zdialog window positions
+int  zdialog_geometry(cchar *action);                                            //  load/save zdialog window positiion/size
 void zdialog_set_position(zdialog *zd, cchar *posn);                             //  set initial/new zdialog window position
 void zdialog_save_position(zdialog *zd);                                         //  save zdialog window position
 
@@ -573,13 +597,30 @@ void window_to_mouse(GtkWidget *window);
 
 //  write text report in popup window
 
-GtkWidget * popup_report_open(GtkWidget *paren, cchar *titl, int ww, int hh);    //  open popup text report window
-void popup_report_header(int bold, cchar *format, ...);                          //  write non-scrolling header line
-void popup_report_write(int bold, cchar *format, ...);                           //  write text line
-void popup_report_top();                                                         //  go to top of report window
-void popup_report_bottom();                                                      //  go to bottom of report window
-void popup_report_move(int ww, int hh);                                          //  move window ww/hh pixels
-void popup_report_close(int secs);                                               //  close window in given seconds
+zdialog * popup_report_open(cchar *title, GtkWidget *parent, int ww, int hh,           //  open popup report - pixel size,
+                       int Fheader, textwidget_callbackfunc_t CBfunc, ...);            //    header line, callback function
+void popup_report_header(zdialog *zd, int bold, cchar *format, ...);                   //  write non-scrolling header line
+void popup_report_write(zdialog *zd, int bold, cchar *format, ...);                    //  write text line
+void popup_report_top(zdialog *zd);                                                    //  go to top of report window
+void popup_report_bottom(zdialog *zd);                                                 //  go to bottom of report window
+void popup_report_clear(zdialog *zd);                                                  //  clear report window
+void popup_report_clear(zdialog *zd, int line);                                        //  clear from line to end
+void popup_report_insert(zdialog *zd, int bold, int line, cchar *format, ...);         //  insert new line
+void popup_report_replace(zdialog *zd, int bold, int line, cchar *format, ...);        //  replace existing line
+void popup_report_delete(zdialog *zd, int line);                                       //  delete line
+int  popup_report_find(zdialog *zd, char *matchtext, int line1);                       //  find matching line
+void popup_report_insert_pixbuf(zdialog *zd, int line, GdkPixbuf *pixbuf);             //  insert pixbuf image after line
+void popup_report_scroll(zdialog *zd, int line);                                       //  scroll to make line visible
+void popup_report_scroll_top(zdialog *zd, int line);                                   //  scroll to put line at top
+void popup_report_get_visible_lines(zdialog *zd, int &top, int &bott);                 //  get visible lines range
+char * popup_report_line(zdialog *zd, int line, int strip);                            //  retrieve line (strip \n) 
+char * popup_report_word(zdialog *zd, int line, int posn, cchar *dlims, char &end);    //  retrieve word
+void popup_report_highlight_line(zdialog *zd, int line);                               //  highlight line
+void popup_report_highlight_word(zdialog *zd, int line, int posn, int cc);             //  highlight word
+void popup_report_underline_word(zdialog *zd, int line, int posn, int cc);             //  underline word 
+void popup_report_bold_word(zdialog *zd, int line, int posn, int cc);                  //  bold word
+void popup_report_font_attributes(zdialog *zd);                                        //  font attributes for entire report
+void popup_report_close(zdialog *zd, int secs);                                        //  close window after seconds
 
 //  shell command to popup window
 
@@ -600,10 +641,11 @@ void poptext_widget(GtkWidget *, cchar *tx, int x, int y, float s1, float s2);
 int  poptext_killnow();                                                          //  kill current popup window
 int  popup_image(cchar *imagefile, GtkWindow *parent, int Fnewin, int size);     //  popup window with image
 
-//  file chooser dialogs for one file or multiple files
+//  file chooser dialogs for one file, multiple files, or folder
 
 char  * zgetfile(cchar *title, GtkWindow *parent, cchar *action, cchar *file, int hidden = 0);
 char ** zgetfiles(cchar *title, GtkWindow *parent, cchar *action, cchar *file, int hidden = 0);
+char  * zgetfolder(cchar *title, GtkWindow *parent, cchar *initfolder);
 
 //  print an image file, choosing printer, paper, orientation, margins, and scale
 
@@ -683,8 +725,8 @@ public:
 
 class HashTab
 {
-   static int trys1;                                                             //  insert trys
-   static int trys2;                                                             //  find/delete trys
+   static int tries1;                                                            //  insert tries
+   static int tries2;                                                            //  find/delete tries
    int     cap;                                                                  //  table capacity
    int     count;                                                                //  strings contained
    int     cc;                                                                   //  string length
@@ -711,8 +753,6 @@ class  Queue
    int         qcount;                                                           //  curr. queue count
    int         ent1;                                                             //  first entry pointer
    int         entN;                                                             //  last entry pointer
-   char      * lastent;                                                          //  last entry retrieved
-   int         lcc;                                                              //  last entry cc
 private:
    void lock();                                                                  //  auto locking and unlocking
    void unlock();                                                                //  (for multi-thread access)
